
#####CODING FOR STATISTICAL COMPETITION ***##

##Libraries needed for coding
library(leaps)  ####for subset selection
library(glmnet)  ###GLMS - Ridge and Lasso regression
library(MASS)
library(pls)
library(class)
library(car)
library(boot)
library(randomForest)
library(tree)
library(splines)
library(gam)
library(e1071)
library(ROCR)
library(pROC)
library(gbm)
library(caret)
library(nnet)
library(neuralnet)
library(h2o)
library(ISLR)
library(lars)
library(ipred)
library(rda)
library(data.table)
library(rms)

###LOADING DATA INTO R 
credit=read.csv("D:/Statistical Competition/Credit card related/Project/credit_card.csv", header = T)
attach(credit)
Y=as.factor(Y)
summary(as.factor(Y))

###PRELIMINARY ANALYSIS to check for important variables
lm.fit1=lm(Y~.,data=credit)
summary(lm.fit1)
#summary(lm.fit1, round(coef(lm.fit1),2))  ###preliminary analysis result includes correlation coefficient
lm.fit2=lm(Y~.-X9-X10-X11-X13-X14-X15-X16-X17-X19-X20-X21-X22-X23, data=credit)
summary(lm.fit2)
anova(lm.fit2,lm.fit1) 

###STEPWISE SELECTION ALGORITHM

####Forward Stepwise Selection####
fit.fwd<-regsubsets(Y~.,data=credit, nvmax=23,method="forward") 
sum1=summary(fit.fwd)
sum1
plot(fit.fwd,scale="adjr2", main="Forward selection based on Adjusted R-squared")
nvmax=which.max(sum1$adjr2); nvmax

fit.fwd2<-regsubsets(Y~.,data=credit, nvmax=which.max(sum1$adjr2),method="forward") 
summary(fit.fwd2)
plot(fit.fwd2,scale="adjr2")


####Backward Stepwise Selection####
fit.bwd<-regsubsets(Y~.,data=credit, nvmax=23,method="backward") 
sum2=summary(fit.bwd)
sum2
plot(fit.bwd,scale="adjr2",main="Backward selection based on Adjusted R-squared")
which.max(sum2$adjr2)

fit.bwd2<-regsubsets(Y~.,data=credit, nvmax=which.max(sum2$adjr2),method="backward") 
summary(fit.bwd2)
plot(fit.bwd2,scale="adjr2")


####Best Subset Selection####
fit.best<-regsubsets(Y~.,data=credit, nvmax=23,method="exhaustive")
sum3=summary(fit.best)
sum3
plot(fit.best,scale="adjr2",main="Best subset selection based on Adjusted R-squared")
which.max(sum3$adjr2) 

fit.best2<-regsubsets(Y~.,data=credit, nvmax=which.max(sum3$adjr2),method="exhaustive") 
summary(fit.best2)
plot(fit.best2,scale="adjr2")


###SPLITTING THE DATA INTO TRAINING AND TESTING AND/OR VALIDATION
dim(credit)
n=length(Y); n
set.seed(1)
train=sample(1:n,n/2)  ##randomly select half numbers for train
test=(-train)          ##left for testing

credit.x=credit[,-1]   ##Predictors
train.X=credit.x[-test,]
test.X=credit.x[test,]

Y=credit[,1]          ####Response
train.Y=Y[-test]
test.Y=Y[test]


#############################USING THE FULL MODEL WITH ALL 23 PREDICTORS#####################################

#################LOGISTIC REGRESSION##########
set.seed(1)
log.fit=glm(Y~.,data=credit,subset=train,family=binomial)
log.prob=predict(log.fit,credit[test,],type="response")
log.pred = ifelse(log.prob>0.5,1,0)
confusionMatrix(test.Y,log.pred,"1") 

roc.log <- roc(test.Y,log.prob,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="green", percent=T)
roc.log$auc

#########LDA - LINEAR DISCRIMINANT ANALYSIS###################
set.seed(1)
lda.fit=lda(Y~.,data=credit,subset=train)
lda.pred=predict(lda.fit,credit[test,])
confusionMatrix(test.Y,lda.pred$class,"1")

lda.pred=predict(lda.fit,credit[test,])$posterior
lda.prob=lda.pred[,2]
roc.lda <- roc(test.Y,lda.prob,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="red", percent=T)
roc.lda$auc


###########QDA - QUADRATIC DISCRIMINANT ANALYSIS################
set.seed(1)
qda.fit=qda(Y~.,data=credit,subset=train)
qda.pred=predict(qda.fit,credit[test,])
confusionMatrix(test.Y,qda.pred$class,"1")

qda.pred=predict(qda.fit,credit[test,])$posterior
qda.prob=qda.pred[,2]
roc.qda <- roc(test.Y,qda.prob,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.qda$auc


#############RIDGE REGRESSION###########
###create X and Y variables
x.train1=model.matrix(Y~.,data=credit[train,])[,-c(1)] ; 
y.train1=train.Y
x.test1=model.matrix(Y~.,data=credit[test,])[,-c(1)] ; 
y.test1=test.Y

set.seed(1)
ridge.train=glmnet(x.train1,y.train1,alpha=0)
cv.out=cv.glmnet(x.train1,y.train1,alpha=0)
plot(cv.out)
bestlam1=cv.out$lambda.min; bestlam1
lambda.ridge=exp(bestlam1); lambda.ridge

ridge.pred=predict(ridge.train,s=bestlam1,newx=x.test1)
ridge.pred1 = ifelse(ridge.pred>0.5,1,0)
confusionMatrix(y.test1,ridge.pred1,"1")

roc.ridge <- roc(y.test1,ridge.pred,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.ridge$auc


############LASSO REGRESSION##############
set.seed(1)
lasso.train=glmnet(x.train1,y.train1,alpha=1)
cv.out2=cv.glmnet(x.train1,y.train1,alpha=1)
plot(cv.out2)
bestlam2=cv.out2$lambda.min; bestlam2
lambda.lasso=exp(bestlam2); lambda.lasso

lasso.pred=predict(lasso.train,s=bestlam2,newx=x.test1)
predict(lasso.train,s=bestlam2,type="coefficients") ###Detecting zero coefficients
lasso.pred1 = ifelse(lasso.pred>0.5,1,0)
confusionMatrix(y.test1,lasso.pred1,"1") ###changing the postions gives better results

roc.lasso <- roc(y.test1,
               lasso.pred,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.lasso$auc


########KNN - K NEAREST NEIGHBOURS########
##Looping for the best K
credit.knn<-numeric() #Holding variable
i<-seq(from=1,to=150,by=1)  
out<-vector("list", length(i))
for(i in 1:150){
    predict<-knn(train.X,test.X,train.Y,k=i)
    credit.knn<-c(credit.knn,mean(predict==test.Y))
    out[[i]]<-mean(predict==test.Y)
    }
best<-unlist(out)
mtable<-data.frame(i,best)
best.K<-min(subset(mtable,best==max(best))$i); 
best.K=100

##Running the KNN code
set.seed(1)
knn.pred=knn (train.X,test.X,train.Y,k=best.K,prob=T) 
probs.knn<-attr(knn.pred,"prob")

confusionMatrix(test.Y,knn.pred,"1")
roc.knn<- roc(test.Y,
              probs.knn,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
              print.auc=TRUE, show.thres=TRUE,col="green", percent=T)
roc.knn$auc


#############RANDOM FOREST##############
##tuning for best n.trees##
set.seed(1)
m<-seq(from=500,to=10000,by=500)
out<-vector("list", length(m))
for(i in m ) {
  fit<-randomForest(as.factor(Y)~.,data=credit[train,],mtry=5,n.trees=i,importance =TRUE)
  pred<-predict(fit,newdata=test.X)
  out[[i]]<-mean(pred==test.Y) 
}
best<-unlist(out)
mtable<-data.frame(m,best)
best.ntree<-min(subset(mtable,best==max(best))$m)
best.ntree

##tuning for best mtry##
set.seed(1)
m<-seq(from=1,to=6,by=1)  ###chose 6 because square root of p=23 is approximately 5
out<-vector("list", length(m))
for(i in m ) {
  fit<-randomForest(as.factor(Y)~.,data=credit[train,],mtry=i,n.trees=best.ntree,importance =TRUE)
  pred<-predict(fit,newdata=test.X)
  out[[i]]<-mean(pred==test.Y) 
}
best<-unlist(out)
mtable<-data.frame(m,best)
best.mtry<-min(subset(mtable,best==max(best))$m)
best.mtry

#tuneRF(train.X,train.Y,mtryStart = 1,ntreeTry=500) ###best mtry = 64

###use tuned paramters for prediction###
rf.fit<-randomForest(Y~.,data=credit[train,],mtry=best.mtry,n.trees=best.ntree,importance =TRUE)
rf.pred<-predict(rf.fit,newdata=test.X, n.trees=best.ntree)
pred.rf=ifelse(rf.pred>.5,1,0) 
importance(rf.fit); varImpPlot(rf.fit,sort=T)

confusionMatrix(test.Y,pred.rf,"1") 


roc.rf <- roc(test.Y, pred.rf,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
              grid=TRUE, print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.rf$auc


##############BOOSTING - ADABOOST#############
##tuning for n.tree##
set.seed(1)
m<-seq(from=500,to=10000,by=500)
out<-vector("list", length(m))
for(i in m ) {
          fit<-gbm(Y~.,data=credit[train,],distribution = "adaboost",n.trees=i, interaction.depth = 2)
          boost<-predict(fit,newdata=test.X,n.trees=i,type="response")
          pred=ifelse(boost>.5,1,0)
          out[[i]]<-mean(pred==test.Y) 
          }
best<-unlist(out)
mtable<-data.frame(m,best)
best.ntrees.boost<-min(subset(mtable,best==max(best))$m)
best.ntrees.boost

##tuning for best interaction.depth##
set.seed(1)
m<-seq(from=1,to=10,by=1)
out<-vector("list", length(m))
for(i in m ) {
  fit<-gbm(Y~.,data=credit[train,],distribution = "adaboost",n.trees=best.ntrees.boost, interaction.depth = i)
  boost<-predict(fit,newdata=test.X,n.trees=best.ntrees.boost,type="response")
  pred=ifelse(boost>.5,1,0)
  out[[i]]<-mean(pred==test.Y) 
}
best<-unlist(out)
mtable<-data.frame(m,best)
best.int.depth<-min(subset(mtable,best==max(best))$m)
best.int.depth


###adabost prediction###
set.seed(1)
boost.fit<-gbm(Y~.,data=credit[train,],distribution = "adaboost",n.trees=best.ntrees.boost, interaction.depth = best.int.depth, shrinkage=0.2, cv.folds = 10)
summary(boost.fit)
boost.pred=predict(boost.fit,newdata=test.X,n.trees=best.ntrees.boost,type="response")
pred.boost=ifelse(boost.pred>.5,1,0)
#table(pred.boost,y.test)
confusionMatrix(test.Y,pred.boost,"1")

roc.boost <- roc(test.Y,pred.boost,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
                 grid=TRUE, print.auc=TRUE, show.thres=TRUE,col="red", percent=T)
roc.boost$auc


############SUPPORT VECTOR MACHINES###############
set.seed(1)
svmfit.credit=svm(as.factor(Y)~.,data=credit[train,], kernel ="radial", decision.values = TRUE, probability = TRUE)
svm.pred <- predict(svmfit.credit,credit[test,], decision.values = TRUE, probability = TRUE)
predict.svm<-attr(svm.pred, "probabilities")[,2]

summary(svmfit.credit)
confusionMatrix(test.Y,svm.pred,"1")

roc.svm <- roc(test.Y,predict.svm,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
              grid=TRUE, print.auc=TRUE, show.thres=TRUE,col="yellow",percent=T)
roc.svm$auc

############ARTIFICIAL NEURAL NETWORK###############
set.seed(1)
credit.net<-neuralnet(Y~X1+X2+X3+X4+X5+X6+X7+X8+X9+X10+X11+X12+X13+X14+X15+X16+
                        X17+X18+X19+X20+X21+X22+X23,data=credit[train,], linear.output = FALSE, hidden = 2, threshold = 0.01)
pred.net <- compute(credit.net,test.X)
plot(credit.net)
#sort(pred.net$net.result,decreasing = T)[15000] ###Checks the value of the sorted predictors ; 15000th = last number (you can ignore that)
pred.net.level = ifelse(pred.net$net.result>0.2212,1,0)
#table(pred.net.level)
#table(test.Y)
confusionMatrix(test.Y,pred.net.level,"1")

roc.net <- roc(test.Y,pred.net.level,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
               grid=TRUE,print.auc=TRUE, show.thres=TRUE,col="green", percent=T)
roc.net$auc



#######################GRAPHING ALL ROC CURVES IN ONE - Full Model#######################################
roc.log <- roc(test.Y, log.prob, 
               plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE,grid=TRUE, 
               show.thres=TRUE,col="black",lwd=1, main = "ROC Curves for The Full Model")
roc.lda <- roc(test.Y,lda.prob, 
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="red", lwd=1)
roc.qda <- roc(test.Y, qda.prob,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="azure", lwd=1)
roc.ridge <- roc(y.test1,ridge.pred,
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="brown", lwd=1)
roc.lasso <- roc(y.test1,lasso.pred,
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="yellow", lwd=1)
roc.knn<- roc(test.Y,probs.knn,
              plot=TRUE, add=TRUE, percent=roc.log$percent, col="blue", lwd=1)
roc.rf <- roc(test.Y, pred.rf,
              plot=TRUE, add=TRUE, percent=roc.log$percent, col="orange", lwd=1)
roc.boost <- roc(test.Y,pred.boost, 
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="purple", lwd=1)
roc.svm <- roc(test.Y,predict.svm,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="turquoise", lwd=1)
roc.net <- roc(test.Y,pred.net.level,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="gold", lwd=1)
               

legend(0.4,0.55,c('LgR = 72.41%', 'LDA = 71.95%', 'QDA = 72.24%', 'Ridge=71.97%', 'Lasso=71.92%', 
                  'KNN = 65.02%','Ran.F=65.26%','Boost=64.18%', 'SVM = 72.12%', "NNet = 56.39%"),
       col=c('black','red','azure','brown', 'yellow','blue','orange','purple','turquoise', 'gold'),lwd=3)

roc.log$auc
roc.lda$auc
roc.qda$auc
roc.ridge$auc
roc.lasso$auc
roc.knn$auc
roc.rf$auc             
roc.boost$auc
roc.svm$auc
roc.net$auc               







#############################USING THE REDUCED MODEL WITH 10 MOST IMPORTANT PREDICTORS###########################

###Loading the reduced data
credit.reduced=read.csv("D:/Pendrive/Statistical Competition/Credit card related/Project/credit_card2.csv", header = T)
attach(credit.reduced)


###SPLITTING THE REDUCED DATA INTO TRAINING AND TESTING AND/OR VALIDATION
dim(credit.reduced)
n=length(Y); n
set.seed(1)
train2=sample(1:n,n/2)  ##randomly select half numbers for train
test2=(-train2)          ##left for testing

credit.x2=credit.reduced[,-1]  ##Predictors
train.X2=credit.x2[-test2,]
test.X2=credit.x2[test2,]

Y=credit.reduced[,1]          ####Response
train.Y2=Y[-test2]
test.Y2=Y[test2]
y.test2=Y[test2]


#############LOGISTIC REGRESSION - REDUCED###################
set.seed(1)
log.fit2=glm(Y~.,data=credit.reduced,subset=train2,family=binomial)
log.prob2=predict(log.fit2,credit[test2,],type="response")
log.pred2 = ifelse(log.prob>0.5,1,0)
confusionMatrix(test.Y2,log.pred2,"1") 

roc.log2 <- roc(test.Y2,
               log.prob2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="green", percent=T)
roc.log2$auc


################LDA - REDUCED######################
set.seed(1)
lda.fit2=lda(Y~.,data=credit.reduced,subset=train2)
lda.pred2=predict(lda.fit2,credit.reduced[test2,])
confusionMatrix(test.Y2,lda.pred2$class,"1")

lda.pred2=predict(lda.fit2,credit.reduced[test2,])$posterior
lda.prob2=lda.pred2[,2]
roc.lda2 <- roc(test.Y2,
                lda.prob2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
                print.auc=TRUE, show.thres=TRUE,col="red", percent=T)
roc.lda2$auc


################QDA - REDUCED######################
set.seed(1)
qda.fit2=qda(Y~.,data=credit.reduced,subset=train2)
qda.pred2=predict(qda.fit2,credit.reduced[test2,])
confusionMatrix(test.Y2,qda.pred2$class,"1")

qda.pred2=predict(qda.fit2,credit.reduced[test2,])$posterior
qda.prob2=qda.pred2[,2]
roc.qda2 <- roc(test.Y2,
                qda.prob2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
                print.auc=TRUE, show.thres=TRUE,col="red", percent=T)

roc.qda2$auc


#############RIDGE REGRESSION - REDUCED###########
###create X and Y variables
x.train2=model.matrix(Y~.,data=credit.reduced[train2,])[,-c(1)] ; y.train2=train.Y2
x.test2=model.matrix(Y~.,data=credit.reduced[test2,])[,-c(1)] ; y.test2=test.Y2

set.seed(1)
ridge.train2=glmnet(x.train2,y.train2,alpha=0)
cv.out3=cv.glmnet(x.train2,y.train2,alpha=0)
plot(cv.out3)
bestlam3=cv.out3$lambda.min; bestlam3
lambda2.ridge=exp(bestlam3); lambda2.ridge

ridge.pred2=predict(ridge.train2,s=bestlam3,newx=x.test2)
ridge.pred3 = ifelse(ridge.pred2>0.5,1,0)
confusionMatrix(y.test2,ridge.pred3,"1")

roc.ridge2 <- roc(y.test2,
                 ridge.pred2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
                 print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.ridge2$auc


#############LASSO REGRESSION - REDUCED###########
set.seed(1)
lasso.train2=glmnet(x.train2,y.train2,alpha=1)
cv.out4=cv.glmnet(x.train2,y.train2,alpha=1)
plot(cv.out4)
bestlam4=cv.out4$lambda.min; bestlam4
lambda2.lasso=exp(bestlam4); lambda2.lasso

lasso.pred2=predict(lasso.train2,s=bestlam4,newx=x.test2)
lasso.pred3 = ifelse(lasso.pred2>0.5,1,0)
confusionMatrix(y.test2,lasso.pred3,"1")

roc.lasso2 <- roc(y.test2,
                 lasso.pred2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
                 print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.lasso2$auc


########KNN - K NEAREST NEIGHBOURS - REDUCED########
set.seed(1)
knn.pred2=knn (train.X2,test.X2,train.Y2,k=best.K,prob=T)
probs.knn2<-attr(knn.pred2,"prob")

confusionMatrix(test.Y2,knn.pred2,"1")
roc.knn2<- roc(test.Y2,
              probs.knn2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
              print.auc=TRUE, show.thres=TRUE,col="green", percent=T)
roc.knn2$auc


#############RANDOM FOREST - REDUCED##############
##tuning for best n.trees##
set.seed(1)
m2<-seq(from=500,to=10000,by=500)
out<-vector("list", length(m2))
for(i in m2 ) {
  fit<-randomForest(as.factor(Y)~.,data=credit.reduced[train2,],mtry=5,n.trees=i,importance =TRUE)
  pred<-predict(fit,newdata=test.X2)
  out[[i]]<-mean(pred==test.Y2) 
}
best<-unlist(out)
mtable<-data.frame(m2,best)
best.ntree2<-min(subset(mtable,best==max(best))$m2)
best.ntree2

##tuning for best mtry##
set.seed(1)
m<-seq(from=1,to=6,by=1)  ###chose 4 because square root of p=10 is approximately 4 up
out<-vector("list", length(m))
for(i in m ) {
  fit<-randomForest(as.factor(Y)~.,data=credit.reduced[train2,],mtry=i,n.trees=best.ntree2,importance =TRUE)
  pred<-predict(fit,newdata=test.X2)
  out[[i]]<-mean(pred==test.Y2) 
}
best<-unlist(out)
mtable<-data.frame(m,best)
best.mtry2<-min(subset(mtable,best==max(best))$m)
best.mtry2

###Using tuned parameters for prediction
set.seed(1)
rf.fit2<-randomForest(Y~.,data=credit.reduced[train2,],mtry=best.mtry2,n.trees=best.ntree2,importance =TRUE)
rf.pred2<-predict(rf.fit2,data=credit.reduced[test2,], ntrees = best.ntree2)  
pred.rf2=ifelse(rf.pred2>.5,1,0)
confusionMatrix(test.Y2,pred.rf2,"1") 

roc.rf2 <- roc(test.Y2,
               pred.rf2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, grid=TRUE,
               print.auc=TRUE, show.thres=TRUE,col="blue", percent=T)
roc.rf2$auc


#############BOOSTING - ADABOOST - REDUCED##############
###adabost prediction###
set.seed(1)
boost.fit2<-gbm(Y~.,data=credit.reduced[train2,],distribution = "adaboost",n.trees=best.ntrees.boost, interaction.depth = best.int.depth, shrinkage=0.2, cv.folds = 10)
boost.pred2=predict(boost.fit2,newdata=test.X2,n.trees=best.ntrees.boost,type="response")
pred.boost2=ifelse(boost.pred2>.5,1,0)
#table(pred.boost,y.test)
confusionMatrix(test.Y2,pred.boost2,"1")

roc.boost2 <- roc(test.Y2,pred.boost2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
                 grid=TRUE, print.auc=TRUE, show.thres=TRUE,col="red",percent=T)
roc.boost2$auc

############SUPPORT VECTOR MACHINES - REDUCED###############
set.seed(1)
svmfit.credit2=svm(as.factor(Y)~.,data=credit.reduced[train2,], kernel ="radial", decision.values = TRUE, probability = TRUE)
svm.pred2 <- predict(svmfit.credit2,credit.reduced[test2,], decision.values = TRUE, probability = TRUE)
predict.svm2<-attr(svm.pred2, "probabilities")[,2]

summary(svmfit.credit2)
confusionMatrix(test.Y2,svm.pred2,"1")

roc.svm2 <- roc(test.Y2,predict.svm2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
               grid=TRUE, print.auc=TRUE, show.thres=TRUE,col="yellow",percent = T)
roc.svm2$auc


############ARTIFICIAL NEURAL NETWORK - REDUCED###############
set.seed(1)
credit.net2<-neuralnet(Y~X1+X2+X3+X4+X5+X6+X7+X8+X12+X18,
                       data=credit.reduced[train2,], linear.output = FALSE, hidden = 2, threshold = 0.01)
pred.net2 <- compute(credit.net2,test.X2)
plot(credit.net2)
#sort(pred.net$net.result,decreasing = T)[15000] ###Checks the value of the sorted predictors ; 15000th = last number (you can ignore that)
pred.net.level2 = ifelse(pred.net2$net.result>0.2212,1,0)
#table(pred.net.level)
#table(test.Y)
confusionMatrix(test.Y2,pred.net.level2,"1")

roc.net2 <- roc(test.Y2,pred.net.level2,plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE, 
                grid=TRUE,print.auc=TRUE, show.thres=TRUE,col="green",percent = T)
roc.net2$auc




#######################GRAPHING ALL ROC CURVES IN ONE - Full Model#######################################
roc.log2 <- roc(test.Y2, log.prob2, 
               plot=TRUE, auc.polygon=TRUE, max.auc.polygon=TRUE,grid=TRUE, 
               show.thres=TRUE,col="black",lwd=1, main = "ROC Curves for The Reduced Model")
roc.lda2 <- roc(test.Y2,lda.prob2, 
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="red", lwd=1)
roc.qda2 <- roc(test.Y2, qda.prob2,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="azure", lwd=1)
roc.ridge2 <- roc(y.test2, ridge.pred2,
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="brown", lwd=1)
roc.lasso2 <- roc(y.test2, lasso.pred2,
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="yellow", lwd=1)
roc.knn2<- roc(test.Y2, probs.knn2,
              plot=TRUE, add=TRUE, percent=roc.log$percent, col="blue", lwd=1)
roc.rf2 <- roc(test.Y2, pred.rf2,
              plot=TRUE, add=TRUE, percent=roc.log$percent, col="orange", lwd=1)
roc.boost2 <- roc(test.Y2, pred.boost2, 
                 plot=TRUE, add=TRUE, percent=roc.log$percent, col="purple", lwd=1)
roc.svm2 <- roc(test.Y2, predict.svm2,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="turquoise", lwd=1)
roc.net2 <- roc(test.Y2, pred.net.level2,
               plot=TRUE, add=TRUE, percent=roc.log$percent, col="gold", lwd=1)


legend(0.41,0.55,c('LgR2 = 72.41%', 'LDA2 = 72.11%', 'QDA2 = 73.23%', 'Ridge2=72.19%', 'Lasso2=72.11%', 
                  'KNN2 = 62.99%','Ran.F2=50.28%','Boost2=63.55%', 'SVM2 = 71.78%', "NNet2 = 52.05%"),
       col=c('black','red','azure','brown', 'yellow','blue','orange','purple','turquoise', 'gold'),lwd=3)






































